@@ -46,6 +46,7 @@
 #include <assert.h>
 #include <errno.h>
 #include <string.h>
+#include <stdlib.h>
 
 #include <gnutls/gnutls.h>
 #include <gnutls/x509.h>
@@ -79,6 +80,11 @@ void GWENHYWFAR_CB GWEN_SyncIo_Tls_FreeData(void *bp, void *p) {
   GWEN_SYNCIO_TLS *xio;
 
   xio=(GWEN_SYNCIO_TLS*) p;
+  free(xio->localCertFile);
+  free(xio->localKeyFile);
+  free(xio->localTrustFile);
+  free(xio->dhParamFile);
+  free(xio->hostName);
   GWEN_FREE_OBJECT(xio);
 }
 
@@ -256,6 +262,67 @@ int GWEN_SyncIo_Tls__readFile(const char *fname, GWEN_BUFFER *buf) {
 
 
 
+
+#if 0
+static int GWEN_SyncIo_Tls_AddCaCertFolder(GWEN_SYNCIO *sio, const char *folder) {
+  GWEN_SYNCIO_TLS *xio;
+  int rv;
+  int successfullTustFileCount=0;
+
+  assert(sio);
+  xio=GWEN_INHERIT_GETDATA(GWEN_SYNCIO, GWEN_SYNCIO_TLS, sio);
+  assert(xio);
+
+  if (folder && *folder) {
+    GWEN_STRINGLIST *fileList;
+
+    fileList=GWEN_StringList_new();
+    rv=GWEN_Directory_GetMatchingFilesRecursively(folder, fileList, "*.crt");
+    if (rv<0) {
+      DBG_ERROR(GWEN_LOGDOMAIN,
+		"Error reading list of certificate files (%d) in folder [%s]",
+		rv, folder);
+    }
+    else {
+      GWEN_STRINGLISTENTRY *se;
+  
+      se=GWEN_StringList_FirstEntry(fileList);
+      while(se) {
+	const char *s;
+
+	s=GWEN_StringListEntry_Data(se);
+	if (s && *s) {
+	  rv=gnutls_certificate_set_x509_trust_file(xio->credentials,
+						    s,
+						    GNUTLS_X509_FMT_PEM);
+	  if (rv<=0) {
+	    DBG_WARN(GWEN_LOGDOMAIN,
+		     "gnutls_certificate_set_x509_trust_file(%s): %d (%s)",
+		     s, rv, gnutls_strerror(rv));
+	  }
+	  else {
+	    DBG_INFO(GWEN_LOGDOMAIN,
+		     "Added %d trusted certs from [%s]", rv, s);
+	    successfullTustFileCount++;
+	  }
+	}
+
+	se=GWEN_StringListEntry_Next(se);
+      } /* while */
+    }
+    GWEN_StringList_free(fileList);
+  }
+
+  if (successfullTustFileCount==0) {
+    DBG_ERROR(GWEN_LOGDOMAIN, "No files added from folder [%s]", folder);
+  }
+
+  return successfullTustFileCount;
+}
+#endif
+
+
+
 int GWEN_SyncIo_Tls_Prepare(GWEN_SYNCIO *sio) {
   GWEN_SYNCIO_TLS *xio;
   int rv;
@@ -283,6 +350,8 @@ int GWEN_SyncIo_Tls_Prepare(GWEN_SYNCIO *sio) {
   }
 
   /* set default priority */
+#if 1 /* old code */
+  GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error, I18N("Using old SSL preparation code."));
   rv=gnutls_set_default_priority(xio->session);
   if (rv) {
     DBG_ERROR(GWEN_LOGDOMAIN, "gnutls_set_default_priority: %d (%s)", rv, gnutls_strerror(rv));
@@ -302,6 +371,88 @@ int GWEN_SyncIo_Tls_Prepare(GWEN_SYNCIO *sio) {
       return GWEN_ERROR_GENERIC;
     }
   }
+#else /* new code */
+  { /* TODO: The following does not work with all servers, disabled for now  */
+    const char *errPos=NULL;
+    const char *ciphers_default;
+    GWEN_BUFFER *ciphers = NULL;
+
+    GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error, I18N("Using new SSL preparation code."));
+    ciphers=GWEN_Buffer_new(0, 256, 0, 1);
+    assert(ciphers!=NULL);
+
+    ciphers_default=getenv("GWEN_TLS_CIPHER_PRIORITIES");
+    if (ciphers_default && *ciphers_default) { /* use cipher list from env var */
+      rv=GWEN_Buffer_AppendString(ciphers, ciphers_default);
+      if (rv!=0) {
+	DBG_ERROR(GWEN_LOGDOMAIN, "failed to append default ciphers to cipher list: %d", rv);
+	GWEN_Buffer_free(ciphers);
+	gnutls_deinit(xio->session);
+	return GWEN_ERROR_GENERIC;
+      }
+    }
+    else { /* use hardcoded default ciphers */
+
+      /* this might get removed later, because it is no longer needed */
+      if (lflags & GWEN_SYNCIO_TLS_FLAGS_FORCE_SSL_V3) {
+	rv=GWEN_Buffer_AppendString(ciphers, "+VERS-SSL3.0:");
+	if (rv!=0) {
+	  DBG_ERROR(GWEN_LOGDOMAIN, "failed to append SSLv3 prefix to cipher list: %d", rv);
+	  GWEN_Buffer_free(ciphers);
+	  gnutls_deinit(xio->session);
+	  return GWEN_ERROR_GENERIC;
+	}
+      }
+
+      if (lflags & GWEN_SYNCIO_TLS_FLAGS_FORCE_UNSAFE_CIPHERS) {
+	GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error, I18N("TLS: Warning - Forcing unsafe SSL-Ciphers per user request."));
+	/* add hardcoded 128-Bit-ciphers ciphers */
+	rv=GWEN_Buffer_AppendString(ciphers, GWEN_TLS_CIPHER_PRIORITIES_UNSAFE);
+	if (rv!=0) {
+	  DBG_ERROR(GWEN_LOGDOMAIN, "failed to append default ciphers to cipher list: %d", rv);
+	  GWEN_Buffer_free(ciphers);
+	  gnutls_deinit(xio->session);
+	  return GWEN_ERROR_GENERIC;
+	}
+      }
+      else {
+	/* add hardcoded default ciphers */
+	rv=GWEN_Buffer_AppendString(ciphers, GWEN_TLS_CIPHER_PRIORITIES_DEFAULT);
+	if (rv!=0) {
+	  DBG_ERROR(GWEN_LOGDOMAIN, "failed to append default ciphers to cipher list: %d", rv);
+	  GWEN_Buffer_free(ciphers);
+	  gnutls_deinit(xio->session);
+	  return GWEN_ERROR_GENERIC;
+	}
+  
+	/* add list of disabled ciphers */
+	if (lflags & GWEN_SYNCIO_TLS_FLAGS_ONLY_SAFE_CIPHERS) {
+	  DBG_INFO(GWEN_LOGDOMAIN, "Removing unsafe ciphers");
+	  GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Info, I18N("TLS: Disabling unsafe ciphers per user request."));
+	  rv=GWEN_Buffer_AppendString(ciphers, ":"GWEN_TLS_CIPHER_PRIORITIES_DISABLE_UNSAFE);
+	  if (rv!=0) {
+	    DBG_ERROR(GWEN_LOGDOMAIN, "failed to append unsafe ciphers to cipher list: %d", rv);
+	    GWEN_Buffer_free(ciphers);
+	    gnutls_deinit(xio->session);
+	    return GWEN_ERROR_GENERIC;
+	  }
+	}
+      }
+    }
+
+    DBG_INFO(GWEN_LOGDOMAIN, "Setting cipher priority to [%s]", GWEN_Buffer_GetStart(ciphers));
+    GWEN_Gui_ProgressLog2(0, GWEN_LoggerLevel_Info, I18N("TLS: SSL-Cipher priority list: %s"), GWEN_Buffer_GetStart(ciphers));
+    rv=gnutls_priority_set_direct(xio->session, GWEN_Buffer_GetStart(ciphers), &errPos);
+    if (rv!=GNUTLS_E_SUCCESS) {
+      DBG_ERROR(GWEN_LOGDOMAIN, "gnutls_priority_set_direct using '%s' failed: %d (%s) [%s]",
+		GWEN_Buffer_GetStart(ciphers), rv, gnutls_strerror(rv), errPos?errPos:"");
+      GWEN_Buffer_free(ciphers);
+      gnutls_deinit(xio->session);
+      return GWEN_ERROR_GENERIC;
+    }
+    GWEN_Buffer_free(ciphers);
+  }
+#endif
 
   /* protect against too-many-known-ca problem */
   gnutls_handshake_set_max_packet_length(xio->session, 64*1024);
@@ -338,10 +489,9 @@ int GWEN_SyncIo_Tls_Prepare(GWEN_SYNCIO *sio) {
   /* find default trust file if none is selected */
   if (lflags & GWEN_SYNCIO_TLS_FLAGS_ADD_TRUSTED_CAS) {
     int trustFileSet=0;
-
 #if 0
 # ifndef OS_WIN32
-    /* try to find OpenSSL cert file */
+    /* try to find OpenSSL certificates */
     if (trustFileSet==0) {
       GWEN_STRINGLIST *paths;
       GWEN_BUFFER *nbuf;
@@ -358,13 +508,45 @@ int GWEN_SyncIo_Tls_Prepare(GWEN_SYNCIO *sio) {
 	DBG_INFO(GWEN_LOGDOMAIN,
 		 "Using default ca-bundle from [%s]",
 		 GWEN_Buffer_GetStart(nbuf));
-	GWEN_SyncIo_Tls_SetLocalTrustFile(sio, GWEN_Buffer_GetStart(nbuf));
-	trustFileSet=1;
+
+	rv=gnutls_certificate_set_x509_trust_file(xio->credentials,
+						  GWEN_Buffer_GetStart(nbuf),
+						  GNUTLS_X509_FMT_PEM);
+	if (rv<=0) {
+	  DBG_WARN(GWEN_LOGDOMAIN,
+		   "gnutls_certificate_set_x509_trust_file(%s): %d (%s)",
+		   GWEN_Buffer_GetStart(nbuf), rv, gnutls_strerror(rv));
+	}
+	else {
+	  DBG_INFO(GWEN_LOGDOMAIN,
+		   "Added %d trusted certs from [%s]", rv, GWEN_Buffer_GetStart(nbuf));
+	  trustFileSet=1;
+	}
       }
+      GWEN_Buffer_free(nbuf);
     }
 # endif
+
+
+# ifndef OS_WIN32
+    /* try to find ca-certificates (at least available on Debian systems) */
+    if (trustFileSet==0) {
+      rv=GWEN_Directory_GetPath("/usr/share/ca-certificates", GWEN_PATH_FLAGS_NAMEMUSTEXIST);
+      if (rv>=0) {
+	rv=GWEN_SyncIo_Tls_AddCaCertFolder(sio, "/usr/share/ca-certificates");
+	if (rv<=0) {
+	  DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+	}
+	else {
+	  trustFileSet=1;
+	}
+      }
+    }
+
+# endif
 #endif
 
+
     if (trustFileSet==0) {
       GWEN_STRINGLIST *paths;
 
@@ -382,8 +564,19 @@ int GWEN_SyncIo_Tls_Prepare(GWEN_SYNCIO *sio) {
 	  DBG_INFO(GWEN_LOGDOMAIN,
 		   "Using default ca-bundle from [%s]",
 		   GWEN_Buffer_GetStart(nbuf));
-	  GWEN_SyncIo_Tls_SetLocalTrustFile(sio, GWEN_Buffer_GetStart(nbuf));
-	  trustFileSet=1;
+	  rv=gnutls_certificate_set_x509_trust_file(xio->credentials,
+						    GWEN_Buffer_GetStart(nbuf),
+						    GNUTLS_X509_FMT_PEM);
+	  if (rv<=0) {
+	    DBG_ERROR(GWEN_LOGDOMAIN,
+		      "gnutls_certificate_set_x509_trust_file(%s): %d (%s)",
+		      GWEN_Buffer_GetStart(nbuf), rv, gnutls_strerror(rv));
+	  }
+	  else {
+	    DBG_INFO(GWEN_LOGDOMAIN,
+		     "Added %d trusted certs", rv);
+	    trustFileSet=1;
+	  }
 	}
 	GWEN_Buffer_free(nbuf);
       }
@@ -470,7 +663,13 @@ int GWEN_SyncIo_Tls_Prepare(GWEN_SYNCIO *sio) {
   gnutls_transport_set_ptr(xio->session, (gnutls_transport_ptr_t)sio);
   gnutls_transport_set_push_function(xio->session, GWEN_SyncIo_Tls_Push);
   gnutls_transport_set_pull_function(xio->session, GWEN_SyncIo_Tls_Pull);
-#if GNUTLS_VERSION_NUMBER < 0x030003
+#if GNUTLS_VERSION_NUMBER < 0x020c00
+  /* This function must be set to 0 in GNUTLS versions < 2.12.0 because we use
+   * custom push/pull functions.
+   * In GNUTLS 2.12.x this is set to 0 and since version 3 this functions is removed
+   * completely.
+   * So we only call this function now for GNUTLS < 2.12.0.
+   */
   gnutls_transport_set_lowat(xio->session, 0);
 #endif
 
@@ -918,6 +1117,92 @@ ssize_t GWEN_SyncIo_Tls_Push(gnutls_transport_ptr_t p, const void *buf, size_t l
 
 
 
+void GWEN_SyncIo_Tls_ShowCipherInfo(GWEN_SYNCIO *sio) {
+  GWEN_SYNCIO_TLS *xio;
+  const char *s;
+  gnutls_kx_algorithm_t kx;
+  GWEN_BUFFER *cbuf;
+  GWEN_BUFFER *sbuf;
+
+  assert(sio);
+  xio=GWEN_INHERIT_GETDATA(GWEN_SYNCIO, GWEN_SYNCIO_TLS, sio);
+  assert(xio);
+
+  cbuf=GWEN_Buffer_new(0, 256, 0, 1);
+  sbuf=GWEN_Buffer_new(0, 256, 0, 1);
+
+  /* protocol */
+  s=gnutls_protocol_get_name(gnutls_protocol_get_version(xio->session));
+  if (s && *s) {
+    if (GWEN_Buffer_GetUsedBytes(cbuf))
+      GWEN_Buffer_AppendString(cbuf, " ");
+    GWEN_Buffer_AppendString(cbuf, "Protocol: ");
+    GWEN_Buffer_AppendString(cbuf, s);
+
+    GWEN_Buffer_AppendString(sbuf, s);
+  }
+  GWEN_Buffer_AppendString(sbuf, ":");
+
+  /* key exchange algorithm */
+  kx=gnutls_kx_get(xio->session);
+  s=gnutls_kx_get_name(kx);
+  if (s && *s) {
+    if (GWEN_Buffer_GetUsedBytes(cbuf))
+      GWEN_Buffer_AppendString(cbuf, " ");
+    GWEN_Buffer_AppendString(cbuf, "Key exchange algorithm: ");
+    GWEN_Buffer_AppendString(cbuf, s);
+    GWEN_Buffer_AppendString(sbuf, s);
+  }
+  GWEN_Buffer_AppendString(sbuf, "-");
+
+  /* cipher */
+  s=gnutls_cipher_get_name(gnutls_cipher_get(xio->session));
+  if (s && *s) {
+    if (GWEN_Buffer_GetUsedBytes(cbuf))
+      GWEN_Buffer_AppendString(cbuf, " ");
+    GWEN_Buffer_AppendString(cbuf, "cipher algorithm: ");
+    GWEN_Buffer_AppendString(cbuf, s);
+    GWEN_Buffer_AppendString(sbuf, s);
+  }
+  GWEN_Buffer_AppendString(sbuf, ":");
+
+  /* MAC algorithm */
+  s=gnutls_mac_get_name(gnutls_mac_get(xio->session));
+  if (s && *s) {
+    if (GWEN_Buffer_GetUsedBytes(cbuf))
+      GWEN_Buffer_AppendString(cbuf, " ");
+    GWEN_Buffer_AppendString(cbuf, "MAC algorithm: ");
+    GWEN_Buffer_AppendString(cbuf, s);
+    GWEN_Buffer_AppendString(sbuf, s);
+  }
+
+
+  DBG_NOTICE(GWEN_LOGDOMAIN, "%s", GWEN_Buffer_GetStart(cbuf));
+  GWEN_Gui_ProgressLog2(0, GWEN_LoggerLevel_Info, I18N("TLS: SSL-Ciphers negotiated: %s"), GWEN_Buffer_GetStart(sbuf));
+  GWEN_Buffer_free(cbuf);
+  GWEN_Buffer_free(sbuf);
+
+  /* possibly show warning */
+  switch(gnutls_cipher_get(xio->session)) {
+  case GNUTLS_CIPHER_ARCFOUR_128:
+  case GNUTLS_CIPHER_3DES_CBC:
+  case GNUTLS_CIPHER_AES_128_CBC:
+  case GNUTLS_CIPHER_ARCFOUR_40:
+  case GNUTLS_CIPHER_CAMELLIA_128_CBC:
+    GWEN_Gui_ProgressLog(0, GWEN_LoggerLevel_Error, I18N("TLS: Warning - The server has chosen unsafe SSL-Ciphers!"));
+    break;
+  case GNUTLS_CIPHER_AES_256_CBC:
+  case GNUTLS_CIPHER_CAMELLIA_256_CBC:
+  case GNUTLS_CIPHER_RC2_40_CBC:
+  case GNUTLS_CIPHER_DES_CBC:
+  case GNUTLS_CIPHER_AES_192_CBC:
+  default:
+    break;
+  }
+}
+
+
+
 int GWENHYWFAR_CB GWEN_SyncIo_Tls_Connect(GWEN_SYNCIO *sio) {
   GWEN_SYNCIO_TLS *xio;
   GWEN_SYNCIO *baseIo;
@@ -982,6 +1267,9 @@ int GWENHYWFAR_CB GWEN_SyncIo_Tls_Connect(GWEN_SYNCIO *sio) {
     return GWEN_ERROR_SSL;
   }
   else {
+    /* show session info */
+    GWEN_SyncIo_Tls_ShowCipherInfo(sio);
+
     /* check certificate */
     GWEN_SyncIo_SubFlags(sio, GWEN_SYNCIO_TLS_FLAGS_SECURE);
     rv=GWEN_SyncIo_Tls_GetPeerCert(sio);
