@@ -356,6 +356,26 @@ void Typemaker2_Type_SetDupFlags(TYPEMAKER2_TYPE *ty, uint32_t i) {
 
 
 
+uint32_t Typemaker2_Type_GetCopyFlags(const TYPEMAKER2_TYPE *ty) {
+  assert(ty);
+  assert(ty->refCount);
+
+  if (ty->copyFlags==0 && ty->extendsPtr)
+    return Typemaker2_Type_GetCopyFlags(ty->extendsPtr);
+
+  return ty->copyFlags;
+}
+
+
+
+void Typemaker2_Type_SetCopyFlags(TYPEMAKER2_TYPE *ty, uint32_t i) {
+  assert(ty);
+  assert(ty->refCount);
+  ty->copyFlags=i;
+}
+
+
+
 int Typemaker2_Type_GetAccess(const TYPEMAKER2_TYPE *ty) {
   assert(ty);
   assert(ty->refCount);
@@ -376,6 +396,22 @@ void Typemaker2_Type_SetAccess(TYPEMAKER2_TYPE *ty, int i) {
 
 
 
+int Typemaker2_Type_GetUsePrivateConstructor(const TYPEMAKER2_TYPE *ty){
+  assert(ty);
+  assert(ty->refCount);
+  return ty->usePrivateConstructor;
+}
+
+
+
+void Typemaker2_Type_SetUsePrivateConstructor(TYPEMAKER2_TYPE *ty, int i){
+  assert(ty);
+  assert(ty->refCount);
+  ty->usePrivateConstructor=i;
+}
+
+
+
 const char *Typemaker2_Type_GetDefaultValue(const TYPEMAKER2_TYPE *ty) {
   assert(ty);
   assert(ty->refCount);
@@ -959,7 +995,12 @@ int Typemaker2_Type_readXml(TYPEMAKER2_TYPE *ty, GWEN_XMLNODE *node, const char
     s=GWEN_XMLNode_GetCharValue(n, "dupflags", NULL);
     if (s && *s)
       Typemaker2_Type_SetDupFlags(ty, Typemaker2_FlagsFromString(s));
-  
+
+    /* read copyflags */
+    s=GWEN_XMLNode_GetCharValue(n, "copyflags", NULL);
+    if (s && *s)
+      Typemaker2_Type_SetCopyFlags(ty, Typemaker2_FlagsFromString(s));
+
     /* read access */
     s=GWEN_XMLNode_GetCharValue(n, "access", NULL);
     if (s && *s) {
@@ -972,6 +1013,11 @@ int Typemaker2_Type_readXml(TYPEMAKER2_TYPE *ty, GWEN_XMLNODE *node, const char
     }
   }
 
+  s=GWEN_XMLNode_GetProperty(langNode, "usePrivateConstructor", "0");
+  if (s && *s)
+    Typemaker2_Type_SetUsePrivateConstructor(ty, atoi(s));
+
+
   /* read description */
   n=GWEN_XMLNode_FindFirstTag(node, "descr", NULL, NULL);
   if (n) {
@@ -1041,6 +1087,9 @@ void Typemaker2_Type_Dump(TYPEMAKER2_TYPE *ty, FILE *f, int indent) {
     for (i=0; i<indent+2; i++) fprintf(f, " ");
     fprintf(f, "DupFlags  : %08x [%08x]\n", ty->dupFlags, Typemaker2_Type_GetDupFlags(ty));
 
+    for (i=0; i<indent+2; i++) fprintf(f, " ");
+    fprintf(f, "CopyFlags : %08x [%08x]\n", ty->copyFlags, Typemaker2_Type_GetCopyFlags(ty));
+
     for (i=0; i<indent+2; i++) fprintf(f, " ");
     s1=ty->defaultValue;
     s2=Typemaker2_Type_GetDefaultValue(ty);
